@@ -8,7 +8,7 @@ from math import floor
 import os
 from pathlib import Path
 import re
-from typing import Any, List, Optional, Sequence, Tuple, Union
+from typing import Any, List, Literal, Optional, Sequence, Tuple, Union
 import warnings
 
 import numpy as np
@@ -25,6 +25,7 @@ from .._info import BAD_CHANNELS
 from .._text import n_of
 from .._utils import ui, as_list
 from ..mne_fixes import MNE_EVOKED, MNE_RAW, MNE_VOLUME_STC
+from ..mne_fixes._channels import _adjacency_id
 
 
 try:  # mne >= 0.19
@@ -43,9 +44,15 @@ except ImportError:
     }
 
 BaselineArg = Optional[Tuple[Optional[float], Optional[float]]]
+DataArg = Literal['eeg', 'mag', 'grad']
 PicksArg = Any
 
 
+def mne_neighbor_files():
+    neighbor_dir = Path(mne.channels.__file__).parent / 'data' / 'neighbors'
+    return [re.match(r'([\w-]+)_neighb', path.stem).group(1) for path in neighbor_dir.glob('*_neighb.mat')]
+
+
 def mne_raw(path=None, proj=False, **kwargs):
     """Load a :class:`mne.io.Raw` object
 
@@ -119,17 +126,27 @@ def mne_raw(path=None, proj=False, **kwargs):
     return raw
 
 
-def events(raw=None, merge=None, proj=False, name=None, bads=None,
-           stim_channel=None, events=None, annotations=None, **kwargs):
+def events(
+        raw: Union[PathArg, mne.io.BaseRaw] = None,
+        merge: int = None,
+        proj: Union[bool, str] = False,
+        name: str = None,
+        bads: List = None,
+        stim_channel: Union[str, Sequence[str]] = None,
+        events: str = None,
+        annotations: bool = None,
+        feature: Literal['onset', 'offset', 'step'] = 'onset',
+        **kwargs,
+) -> Dataset:
     """
     Load events from a raw fiff file.
 
     Parameters
     ----------
-    raw : str(path) | None | mne Raw
+    raw
         The raw fiff file from which to extract events (if raw and events are
         both ``None``, a file dialog will be displayed to select a raw file).
-    merge : int
+    merge
         Merge stimulus channel steps occurring in neighboring samples.
         The integer value indicates across how many samples events should
         be merged, and the sign indicates in which direction they should be
@@ -141,28 +158,28 @@ def events(raw=None, merge=None, proj=False, name=None, bads=None,
         event onsets can be blurred (what should be ``0, 0, 0, 1, 1, 1, ...``,
         can look like ``0, 0, 0, 0.5, 1, 1, ...``; ``merge=-1`` would
         turn the latter into the former).
-    proj : bool | str
+    proj
         Path to the projections file that will be loaded with the raw file.
         ``'{raw}'`` will be expanded to the raw file's path minus extension.
         With ``proj=True``, ``'{raw}_*proj.fif'`` will be used,
         looking for any projection file starting with the raw file's name.
         If multiple files match the pattern, a ValueError will be raised.
-    name : str | None
+    name
         A name for the Dataset. If ``None``, the raw filename will be used.
-    bads : None | list
+    bads
         Specify additional bad channels in the raw data file (these are added
         to the ones that are already defined in the raw file).
-    stim_channel : None | string | list of string
+    stim_channel
         Name of the stim channel or all the stim channels
         affected by the trigger. If None, the config variables
         'MNE_STIM_CHANNEL', 'MNE_STIM_CHANNEL_1', 'MNE_STIM_CHANNEL_2',
         etc. are read. If these are not found, it will default to
         'STI 014'.
-    events : None | str
+    events
         If events are stored in a fiff file separate from the Raw object, the
         path to the events file can be supplied here. The events in the Dataset
         will reflect the event sin the events file rather than the raw file.
-    annotations : bool
+    annotations
         Generate events from annotations instead of the stim channel (by
         default, annotations are used when present).
     **
@@ -171,7 +188,7 @@ def events(raw=None, merge=None, proj=False, name=None, bads=None,
 
     Returns
     -------
-    events : Dataset
+    events
         A Dataset with the following variables:
          - *i_start*: the index of the event in the raw file.
          - *trigger*: the event value/id.
@@ -183,7 +200,7 @@ def events(raw=None, merge=None, proj=False, name=None, bads=None,
     if (raw is None and events is None) or isinstance(raw, (str, Path)):
         raw = mne_raw(raw, proj=proj, **kwargs)
 
-    if bads is not None and raw is not None :
+    if bads is not None and raw is not None:
         raw.info['bads'].extend(bads)
 
     if name is None and raw is not None:
@@ -202,26 +219,43 @@ def events(raw=None, merge=None, proj=False, name=None, bads=None,
                 annotations = raw.annotations[index]
 
         if annotations:
-            evts, event_ids = mne.events_from_annotations(raw)
+            events, event_ids = mne.events_from_annotations(raw)
         else:
-            raw.load_data()
             if merge is None:
                 if raw.info.get('kit_system_id') is None:
                     merge = 0
                 else:
                     merge = -1
-            evts = mne.find_stim_steps(raw, merge=merge, stim_channel=stim_channel)
-            evts = evts[np.flatnonzero(evts[:, 2])]
+            # Use mne.find_stim_steps() becaue mne.find_events() merges forward (i.e. coerces merge to be ≥ 0); loop through channels because find_stim_steps() only uses the first channel
+            if stim_channel is None:
+                stim_channels = [raw.ch_names[i] for i in mne.pick_types(raw.info, stim=True)]
+            elif isinstance(stim_channel, str):
+                stim_channels = [stim_channel]
+            else:
+                stim_channels = stim_channel
+
+            event_list = []
+            for channel_i in stim_channels:
+                events = mne.find_stim_steps(raw, merge=merge, stim_channel=channel_i)
+                if feature == 'onset':
+                    events = events[np.flatnonzero(events[:, 2])]
+                elif feature == 'offset':
+                    events = events[~np.flatnonzero(events[:, 2])]
+                elif feature != 'step':
+                    raise ValueError(f"{feature=}")
+                event_list.append(events)
+            events = np.concatenate(event_list, axis=0)
+            events = events[np.argsort(events[:, 0])]
             event_ids = getattr(raw, 'event_id', None)
 
         if event_ids:
             labels = {event_id: label for label, event_id in event_ids.items()}
     else:
-        evts = mne.read_events(events)
+        events = mne.read_events(events)
 
     ds = Dataset({
-        'i_start': Var(evts[:, 0]),
-        'trigger': Var(evts[:, 2]),
+        'i_start': Var(events[:, 0]),
+        'trigger': Var(events[:, 2]),
     }, name, info={'raw': raw})
     if labels is not None:
         ds['event'] = Factor(ds['trigger'], labels=labels)
@@ -336,72 +370,101 @@ def _sensor_info(data, vmax, mne_info, user_info=None, mult=1):
     return info
 
 
-def epochs(ds, tmin=-0.1, tmax=None, baseline=None, decim=1, mult=1, proj=False,
-           data=None, reject=None, exclude='bads', info=None, name=None,
-           raw=None, sensors=None, i_start='i_start', tstop=None, sysname=None):
+def epochs(
+        ds: Dataset,
+        tmin: float = -0.1,
+        tmax: float = None,
+        baseline: BaselineArg = None,
+        decim: int = 1,
+        mult: float = 1,
+        proj: bool = False,
+        data: DataArg = None,
+        reject: float = None,
+        exclude: Union[str, Sequence[str]] = 'bads',
+        info: dict = None,
+        name: str = None,
+        raw: mne.io.BaseRaw = None,
+        sensors: Sensor = None,
+        i_start: str = 'i_start',
+        tstop: float = None,
+        sysname: str = None,
+        connectivity: Union[str, Sequence] = None,
+) -> NDVar:
     """
     Load epochs as :class:`NDVar`.
 
     Parameters
     ----------
-    ds : Dataset
+    ds
         Dataset containing a variable which defines epoch cues (i_start).
-    tmin : scalar
+    tmin
         First sample to include in the epochs in seconds (Default is -0.1).
-    tmax : scalar
+    tmax
         Last sample to include in the epochs in seconds (Default 0.6; use
         ``tstop`` instead to specify index exclusive of last sample).
-    baseline : (float, float) | None
+    baseline
         Time interval for baseline correction. ``(tmin, tmax)`` tuple in
         seconds, or ``None`` to use all the data (e.g., ``(None, 0)`` uses all
         the data from the beginning of the epoch up to ``t = 0``). Set to
         ``None`` for no baseline correction (default).
-    decim : int
+    decim
         Downsample the data by this factor when importing. ``1`` means no
         downsampling. Note that this function does not low-pass filter
         the data. The data is downsampled by picking out every
         n-th sample (see `Wikipedia <http://en.wikipedia.org/wiki/Downsampling>`_).
-    mult : scalar
+    mult
         multiply all data by a constant.
-    proj : bool
+    proj
         mne.Epochs kwarg (subtract projections when loading data)
-    data : 'eeg' | 'mag' | 'grad'
+    data
         Which data channels data to include (default based on channels in data).
-    reject : None | scalar
+    reject
         Threshold for rejecting epochs (peak to peak). Requires a for of
         mne-python which implements the Epochs.model['index'] variable.
-    exclude : list of string | str
+    exclude
         Channels to exclude (:func:`mne.pick_types` kwarg).
         If 'bads' (default), exclude channels in info['bads'].
         If empty do not exclude any.
-    info : None | dict
+    info
         Entries for the ndvar's info dict.
-    name : str
+    name
         name for the new NDVar.
-    raw : None | mne Raw
+    raw
         Raw file providing the data; if ``None``, ``ds.info['raw']`` is used.
-    sensors : None | Sensor
+    sensors
         The default (``None``) reads the sensor locations from the fiff file.
         If the fiff file contains incorrect sensor locations, a different
         Sensor instance can be supplied through this kwarg.
-    i_start : str
+    i_start
         name of the variable containing the index of the events.
-    tstop : scalar
-        Alternative to ``tmax``: While ``tmax`` specifies the last samples to 
-        include, ``tstop`` can be used to specify the epoch time excluding the 
-        last time point (i.e., standard Python/Eelbrain indexing convention).
+    tstop
+        Alternative to ``tmax``. While ``tmax`` specifies the last samples to
+        include, ``tstop`` specifies the sample before which to stop (standard
+        Python indexing convention).
         For example, at 100 Hz the epoch with ``tmin=-0.1, tmax=0.4`` will have 
         51 samples, while the epoch specified with ``tmin=-0.1, tstop=0.4`` will
         have 50 samples.
-    sysname : str
+    sysname
         Name of the sensor system to load sensor connectivity (e.g. 'neuromag',
         inferred automatically for KIT data converted with a recent version of
         MNE-Python).
+    connectivity
+        Connectivity between elements. Can be specified as:
+
+        - ``"none"`` for no connections
+        - list of connections (e.g., ``[('OZ', 'O1'), ('OZ', 'O2'), ...]``)
+        - :class:`numpy.ndarray` of int, shape (n_edges, 2), to specify
+          connections in terms of indices. Each row should specify one
+          connection [i, j] with i < j. If the array's dtype is uint32,
+          property checks are disabled to improve efficiency.
+        - ``"grid"`` to use adjacency in the sensor names
+
+        If unspecified, it is inferred from ``sysname`` if possible.
 
     Returns
     -------
-    epochs : NDVar
-        The epochs as NDVar object.
+    epochs
+        The data epochs as (case, sensor, time) data.
     """
     if raw is None:
         raw = ds.info['raw']
@@ -411,10 +474,8 @@ def epochs(ds, tmin=-0.1, tmax=None, baseline=None, decim=1, mult=1, proj=False,
     picks = _picks(raw.info, data, exclude)
     reject = _ndvar_epochs_reject(data, reject)
 
-    epochs_ = mne_epochs(ds, tmin, tmax, baseline, i_start, raw, decim=decim,
-                         picks=picks, reject=reject, proj=proj, tstop=tstop)
-    ndvar = epochs_ndvar(epochs_, name, data, mult=mult, info=info,
-                         sensors=sensors, sysname=sysname)
+    epochs_ = mne_epochs(ds, tmin, tmax, baseline, i_start, raw, decim=decim, picks=picks, reject=reject, proj=proj, tstop=tstop)
+    ndvar = epochs_ndvar(epochs_, name, data, 'bads', mult, info, sensors, None, sysname, connectivity)
 
     if len(epochs_) == 0:
         raise RuntimeError(f"No events left in {raw.filenames[0]}")
@@ -597,9 +658,9 @@ def mne_epochs(ds, tmin=-0.1, tmax=None, baseline=None, i_start='i_start',
     picks, reject
         :class:`mne.Epochs` parameters.
     tstop : scalar
-        Alternative to ``tmax``: While ``tmax`` specifies the last samples to 
-        include, ``tstop`` can be used to specify the epoch time excluding the 
-        last time point (i.e., standard Python/Eelbrain indexing convention).
+        Alternative to ``tmax``. While ``tmax`` specifies the last samples to
+        include, ``tstop`` specifies the sample before which to stop (standard
+        Python indexing convention).
         For example, at 100 Hz the epoch with ``tmin=-0.1, tmax=0.4`` will have 
         51 samples, while the epoch specified with ``tmin=-0.1, tstop=0.4`` will
         have 50 samples.
@@ -668,7 +729,7 @@ def sensor_dim(
         inferred automatically for KIT data converted with a recent version of
         MNE-Python).
     connectivity : str | list of (str, str) | array of int, (n_edges, 2)
-        Connectivity between elements. Can be specified as:
+        Sensor connectivity (adjacency graph). Can be specified as:
 
         - ``"none"`` for no connections
         - list of connections (e.g., ``[('OZ', 'O1'), ('OZ', 'O2'), ...]``)
@@ -688,7 +749,7 @@ def sensor_dim(
     if not isinstance(info, mne.Info):
         info_ = getattr(info, 'info', info)
         if not isinstance(info_, mne.Info):
-            raise TypeError("No mne.Info object: %r" % (info,))
+            raise TypeError(f"{info=}: no mne.Info object")
         info = info_
 
     if picks is None:
@@ -719,23 +780,41 @@ def sensor_dim(
             raise ValueError(f"Unknown channel unit for sysname={sysname!r}: {ch_unit!r}")
 
     if connectivity is None:
-        connectivity = sysname  # or 'auto'
+        connectivity = sysname or 'auto'
+
+    ch_type = None
+    if connectivity == 'auto':
+        if sysname and sysname.startswith('KIT-'):
+            connectivity = sysname
+        else:
+            ch_types = find_mne_channel_types({'chs': chs})
+            if len(ch_types) == 1:
+                ch_type = ch_types[0]
+                if ch_type == 'eog':
+                    connectivity = 'none'
+            else:
+                # TODO: connectivity for vector data
+                connectivity = 'none'
 
     if connectivity in ('grid', 'none'):
         pass
     elif isinstance(connectivity, str):
         if connectivity == 'auto':
-            ch_type = _guess_ndvar_data_type({'chs': chs})
+            connectivity = _adjacency_id(info, ch_type)
+        if connectivity is None:
             c_matrix, names = mne.channels.find_ch_adjacency(info, ch_type)
+            if any(ch not in names for ch in ch_names):
+                raise NotImplementedError("Connectivity fot this data type")
         else:
             c_matrix, names = mne.channels.read_ch_adjacency(connectivity)
+            # fix channel names
             if connectivity.startswith('neuromag'):
                 vec_ids = {name[-1] for name in ch_names}
                 if len(vec_ids) > 1:
                     raise NotImplementedError("Connectivity for Neuromag vector data")
-                vec_id = vec_ids.pop()
-                # fix channel names
-                names = [f'MEG {n[3:-1]}{vec_id}' for n in names]
+                names = [f'{n[:3]} {n[3:]}' for n in names]
+            elif connectivity == 'ctf275':
+                ch_names = [name[:5] for name in ch_names]
 
         # fix channel order
         if names != ch_names:
@@ -749,53 +828,156 @@ def sensor_dim(
     return Sensor(ch_locs, ch_names, sysname, connectivity=connectivity)
 
 
-def variable_length_mne_epochs(
+def variable_length_epochs(
         ds: Dataset,
-        tmin: float,
-        tmax: Sequence[float],
+        tmin: Union[float, Sequence[float]],
+        tmax: Union[float, Sequence[float]] = None,
         baseline: BaselineArg = None,
         allow_truncation: bool = False,
+        data: DataArg = None,
+        exclude: Union[str, Sequence[str]] = 'bads',
+        sysname: str = None,
+        connectivity: Union[str, Sequence] = None,
+        tstop: Union[float, Sequence[float]] = None,
+        name: str = None,
+        **kwargs,
+) -> List[NDVar]:
+    """Load data epochs where each epoch has a different length
+
+    Parameters
+    ----------
+    ds
+        Dataset containing a variable which defines epoch cues (i_start).
+    tmin
+        First sample to include in the epochs in seconds (Default is -0.1).
+    tmax
+        Last sample to include in each epoch in seconds.
+        Use ``tstop`` instead to specify index exclusive of last sample
+    baseline
+        Time interval for baseline correction. ``(tmin, tmax)`` tuple in
+        seconds, or ``None`` to use all the data (e.g., ``(None, 0)`` uses all
+        the data from the beginning of the epoch up to ``t = 0``). Set to
+        ``None`` for no baseline correction (default).
+    allow_truncation
+        If a ``tmax`` value falls outside the data available in ``raw``,
+        automatically truncate the epoch (by default this raises a
+        ``ValueError``).
+    data
+        Which data channels data to include (default based on channels in data).
+    exclude
+        Channels to exclude (:func:`mne.pick_types` kwarg).
+        If 'bads' (default), exclude channels in info['bads'].
+        If empty do not exclude any.
+    sysname
+        Name of the sensor system to load sensor connectivity (e.g. 'neuromag306',
+        inferred automatically for KIT data converted with a recent version of
+        MNE-Python).
+    connectivity : str | list of (str, str) | array of int, (n_edges, 2)
+        Connectivity between elements. Can be specified as:
+
+        - ``"none"`` for no connections
+        - list of connections (e.g., ``[('OZ', 'O1'), ('OZ', 'O2'), ...]``)
+        - :class:`numpy.ndarray` of int, shape (n_edges, 2), to specify
+          connections in terms of indices. Each row should specify one
+          connection [i, j] with i < j. If the array's dtype is uint32,
+          property checks are disabled to improve efficiency.
+        - ``"grid"`` to use adjacency in the sensor names
+
+        If unspecified, it is inferred from ``sysname`` if possible.
+    tstop
+        Alternative to ``tmax``. While ``tmax`` specifies the last samples to
+        include, ``tstop`` specifies the sample before which to stop (standard
+        Python indexing convention).
+        For example, at 100 Hz the epoch with ``tmin=-0.1, tmax=0.4`` will have
+        51 samples, while the epoch specified with ``tmin=-0.1, tstop=0.4`` will
+        have 50 samples.
+    name
+        Name for the NDVar.
+    ...
+        :class:`mne.Epochs` parameters.
+
+    Returns
+    -------
+    epochs
+        List of data epochs of shape.
+    """
+    epochs_ = variable_length_mne_epochs(ds, tmin, tmax, baseline, allow_truncation, tstop=tstop, **kwargs)
+    return [epochs_ndvar(epoch, name, data, exclude, sysname=sysname, connectivity=connectivity)[0] for epoch in epochs_]
+
+
+def variable_length_mne_epochs(
+        ds: Dataset,
+        tmin: Union[float, Sequence[float]],
+        tmax: Union[float, Sequence[float]] = None,
+        baseline: BaselineArg = None,
+        allow_truncation: bool = False,
+        tstop: Union[float, Sequence[float]] = None,
         picks: PicksArg = None,
+        decim: int = 1,
         **kwargs,
 ) -> List[mne.Epochs]:
     """Load mne Epochs where each epoch has a different length
 
     Parameters
     ----------
-    ds : Dataset
+    ds
         Dataset containing a variable which defines epoch cues (i_start).
-    tmin : scalar
+    tmin
         First sample to include in the epochs in seconds (Default is -0.1).
-    tmax : sequence of scalar
+    tmax
         Last sample to include in each epoch in seconds.
-    baseline : (float, float) | None
+    baseline
         Time interval for baseline correction. ``(tmin, tmax)`` tuple in
         seconds, or ``None`` to use all the data (e.g., ``(None, 0)`` uses all
         the data from the beginning of the epoch up to ``t = 0``). Set to
         ``None`` for no baseline correction (default).
-    allow_truncation : bool
+    allow_truncation
         If a ``tmax`` value falls outside the data available in ``raw``,
         automatically truncate the epoch (by default this raises a
         ``ValueError``).
+    tstop
+        Alternative to ``tmax``. While ``tmax`` specifies the last samples to
+        include, ``tstop`` specifies the sample before which to stop (standard
+        Python indexing convention).
+        For example, at 100 Hz the epoch with ``tmin=-0.1, tmax=0.4`` will have
+        51 samples, while the epoch specified with ``tmin=-0.1, tstop=0.4`` will
+        have 50 samples.
     ...
         :class:`mne.Epochs` parameters.
     """
     if baseline is False:
         baseline = None
     raw = ds.info['raw']
+    if tmax is None:
+        if tstop is None:
+            raise TypeError(f"{tmax=}, {tstop=}: must specify at least one")
+        n = len(tstop)
+    else:
+        n = len(tmax)
+    if np.isscalar(tmin):
+        tmin = np.repeat(tmin, n)
+    else:
+        tmin = np.asarray(tmin)
+    if tmax is None:
+        sfreq = raw.info['sfreq'] / decim
+        start_index = np.round(tmin * sfreq).astype(int)
+        stop_index = np.round(np.asarray(tstop) * sfreq).astype(int)
+        tmax = tmin + (stop_index - start_index - 1) / sfreq
+    elif np.isscalar(tmax):
+        tmax = np.repeat(tmax, n)
     if picks is None and raw.info['bads']:
         picks = mne.pick_types(raw.info, meg=True, eeg=True, eog=True, ref_meg=False, exclude=[])
     events = _mne_events(ds)
     out = []
-    for i, tmax_i in enumerate(tmax):
+    for i, (tmin_i, tmax_i) in enumerate(zip(tmin, tmax)):
         i_max = events[i, 0] + floor(tmax_i * raw.info['sfreq'])
         if raw.last_samp < i_max:
             if allow_truncation:
                 tmax_i = (raw.last_samp - events[i, 0]) / raw.info['sfreq']
             else:
                 missing = (i_max - raw.last_samp) / raw.info['sfreq']
-                raise ValueError(f"tmax={tmax}, tmax[{i}] {tmax_i} is outside of data range by {missing:g} s")
-        epochs_i = mne.Epochs(raw, events[i:i+1], None, tmin, tmax_i, baseline, picks, preload=True, **kwargs)
+                raise ValueError(f"{tmax[i]=} is outside of data range by {missing:g} s")
+        epochs_i = mne.Epochs(raw, events[i:i+1], None, tmin_i, tmax_i, baseline, picks, preload=True, decim=decim, **kwargs)
         out.append(epochs_i)
     return out
 
@@ -896,7 +1078,7 @@ def raw_ndvar(raw, i_start=None, i_stop=None, decim=1, data=None, exclude='bads'
 def epochs_ndvar(
         epochs: Union[mne.BaseEpochs, PathArg],
         name: str = None,
-        data: str = None,
+        data: DataArg = None,
         exclude: Union[str, Sequence[str]] = 'bads',
         mult: float = 1,
         info: dict = None,
@@ -904,6 +1086,7 @@ def epochs_ndvar(
         vmax: float = None,
         sysname: str = None,
         connectivity: Union[str, Sequence] = None,
+        proj: bool = True,
 ):
     """
     Convert an :class:`mne.Epochs` object to an :class:`NDVar`.
@@ -914,7 +1097,7 @@ def epochs_ndvar(
         The epochs object or path to an epochs FIFF file.
     name
         Name for the NDVar.
-    data : 'eeg' | 'mag' | 'grad'
+    data
         Which data channels data to include (default based on channels in data).
     exclude
         Channels to exclude (:func:`mne.pick_types` kwarg).
@@ -946,9 +1129,11 @@ def epochs_ndvar(
         - ``"grid"`` to use adjacency in the sensor names
 
         If unspecified, it is inferred from ``sysname`` if possible.
+    proj
+        Add projectors (only applies when ``epochs`` is a path).
     """
     if isinstance(epochs, (str, Path)):
-        epochs = mne.read_epochs(epochs)
+        epochs = mne.read_epochs(epochs, proj=proj)
 
     if data is None:
         data = _guess_ndvar_data_type(epochs.info)
@@ -1084,7 +1269,15 @@ def evoked_ndvar(evoked, name=None, data=None, exclude='bads', vmax=None,
     return NDVar(x, dims, name, info)
 
 
-def forward_operator(fwd, src, subjects_dir=None, parc='aparc', name=None):
+def forward_operator(
+        fwd,
+        src,
+        subjects_dir=None,
+        parc='aparc',
+        sysname: str = None,
+        connectivity: Union[str, Sequence] = None,
+        name=None,
+):
     """Load forward operator as :class:`NDVar`
 
     Parameters
@@ -1112,11 +1305,10 @@ def forward_operator(fwd, src, subjects_dir=None, parc='aparc', name=None):
         if name is None:
             name = os.path.basename(fwd)
         fwd = mne.read_forward_solution(fwd)
-        mne.convert_forward_solution(fwd, force_fixed=not is_vol, use_cps=True,
-                                     copy=False)
+        mne.convert_forward_solution(fwd, force_fixed=not is_vol, use_cps=True, copy=False)
     elif name is None:
         name = 'fwd'
-    sensor = sensor_dim(fwd['info'])
+    sensor = sensor_dim(fwd['info'], sysname=sysname, connectivity=connectivity)
     assert np.all(sensor.names == fwd['sol']['row_names'])
     subject = fwd['src'][0]['subject_his_id']
     if is_vol:
