@@ -20,7 +20,7 @@ x2 = ds['x2']
 from __future__ import annotations
 
 from dataclasses import dataclass, field, fields
-from functools import reduce
+from functools import cached_property, reduce
 import inspect
 from itertools import chain, product, repeat
 from math import ceil
@@ -29,7 +29,7 @@ from operator import mul
 import os
 import time
 from threading import Event, Thread
-from typing import Any, Callable, List, Union, Tuple, Sequence
+from typing import Any, Callable, List, Literal, Union, Tuple, Sequence
 import warnings
 
 import numpy as np
@@ -37,8 +37,8 @@ import numpy as np
 from .._config import CONFIG, mpc
 from .._data_obj import Case, Dataset, Dimension, SourceSpaceBase, NDVar, CategorialArg, NDVarArg, dataobj_repr
 from .._exceptions import OldVersionError
-from .._ndvar import _concatenate_values, convolve_jit, parallel_convolve
-from .._utils import LazyProperty, PickleableDataClass, user_activity
+from .._ndvar import _concatenate_values, convolve_jit, parallel_convolve, set_connectivity, set_parc
+from .._utils import PickleableDataClass, user_activity
 from .._utils.notebooks import tqdm
 from ._boosting_opt import l1, l2, generate_options, update_error
 from .shared import PredictorData, DeconvolutionData, Split, Splits, merge_segments
@@ -230,7 +230,7 @@ class BoostingResult(PickleableDataClass):
                 items.append(f'{name}={value}')
         return f"<{', '.join(items)}>"
 
-    @LazyProperty
+    @cached_property
     def h(self):
         if not self.basis:
             return self._h
@@ -239,7 +239,7 @@ class BoostingResult(PickleableDataClass):
         else:
             return self._h.smooth('time', self.basis, self.basis_window, 'full')
 
-    @LazyProperty
+    @cached_property
     def h_scaled(self):
         if self.y_scale is None:
             return self.h
@@ -255,18 +255,18 @@ class BoostingResult(PickleableDataClass):
                 out.append(h)
             return tuple(out)
 
-    @LazyProperty
+    @cached_property
     def h_source(self):
         return self._h
 
-    @LazyProperty
+    @cached_property
     def h_time(self):
         if isinstance(self.h, NDVar):
             return self.h.time
         else:
             return self.h[0].time
 
-    @LazyProperty
+    @cached_property
     def _variability(self):
         # variability in the data
         if self.y_scale is None:
@@ -388,7 +388,7 @@ class BoostingResult(PickleableDataClass):
             xs = [f'x_{i}' if x is None else x for i, x in enumerate(self.x)]
         return Dataset.from_caselist(['i_test', 'r', 'det', *xs], rows)
 
-    @LazyProperty
+    @cached_property
     def proportion_explained(self):
         return 1 - (self.residual / self._variability)
 
@@ -419,7 +419,11 @@ class BoostingResult(PickleableDataClass):
 
         def func(obj: NDVar):
             return morph_source_space(obj, to_subject)
+        self._apply_ndvar_transform(func)
 
+    def _set_connectivity(self, dim, connectivity):
+        def func(obj: NDVar):
+            return set_connectivity(obj, dim, connectivity)
         self._apply_ndvar_transform(func)
 
     def _set_parc(self, parc: str):
@@ -429,11 +433,8 @@ class BoostingResult(PickleableDataClass):
         -----
         No warning for missing sources!
         """
-        from .._ndvar import set_parc
-
         def func(obj: NDVar):
             return set_parc(obj, parc, mask=True)
-
         self._apply_ndvar_transform(func)
 
     @classmethod
@@ -552,16 +553,20 @@ class Boosting:
         # find TRF start/stop for each x
         if isinstance(tstart, (tuple, list, np.ndarray)):
             if len(tstart) != len(self.data.x_name):
-                raise ValueError(f'tstart={tstart!r}: len(tstart) ({len(tstart)}) is different from len(x) ({len(self.data.x_name)}')
+                raise ValueError(f'{tstart=}: {len(tstart)=} is different from len(x)={len(self.data.x_name)}')
             elif len(tstart) != len(tstop):
-                raise ValueError(f'tstop={tstop!r}: mismatched len(tstart) = {len(tstart)}, len(tstop) = {len(tstop)}')
+                raise ValueError(f'{tstop=}: {len(tstop)=} does not match {len(tstart)=}')
             self.tstart = tuple(tstart)
             self.tstart_h = min(self.tstart)
             self.tstop = tuple(tstop)
+            if any(start >= stop for start, stop in zip(self.tstart, self.tstop)):
+                raise ValueError(f"Some tstart > tstop: {tstart=} and {tstop=}")
             n_xs = [reduce(mul, map(len, xdims), 1) for _, xdims, _ in self.data._x_meta]
             tstart = [t for t, n in zip(tstart, n_xs) for _ in range(n)]
             tstop = [t for t, n in zip(tstop, n_xs) for _ in range(n)]
         else:
+            if tstart >= tstop:
+                raise ValueError(f"{tstart=} > {tstop=}")
             self.tstart = self.tstart_h = tstart
             self.tstop = tstop
             tstart = [tstart] * n_x
@@ -823,7 +828,7 @@ def boosting(
         scale_data: Union[bool, str] = True,  # normalize y and x; can be 'inplace'
         delta: float = 0.005,  # coordinate search step
         mindelta: float = None,  # narrow search by reducing delta until reaching mindelta
-        error: str = 'l2',  # 'l1' | 'l2', for scaling data
+        error: Literal['l1', 'l2'] = 'l2',
         basis: float = 0,
         basis_window: str = 'hamming',
         partitions: int = None,  # Number of partitionings for cross-validation
@@ -834,7 +839,7 @@ def boosting(
         selective_stopping: int = 0,
         partition_results: bool = False,
         debug: bool = False,
-):
+) -> BoostingResult:
     """Estimate a linear filter with coordinate descent
 
     Parameters
@@ -855,13 +860,13 @@ def boosting(
         value (when ``error='l1'``). Use ``'inplace'`` to save memory by scaling
         the original objects specified as ``y`` and ``x`` instead of making a 
         copy.
-    delta : scalar
+    delta
         Step for changes in the kernel.
-    mindelta : scalar
+    mindelta
         If the error for the training data can't be reduced, divide ``delta``
         in half until ``delta < mindelta``. The default is ``mindelta = delta``,
         i.e. ``delta`` is constant.
-    error : 'l2' | 'l1'
+    error
         Error function to use (default is ``l2``).
 
         - ``error='l1'``: the sum of the absolute differences between ``y`` and
@@ -916,17 +921,14 @@ def boosting(
     debug
         Add additional attributes to the returned result.
 
-    Returns
-    -------
-    result : BoostingResult
-        Results (see :class:`BoostingResult`).
-
     See Also
     --------
-    plot.preview_partitions
+    plot.preview_partitions : preview data partitions for cross-validation
 
     Notes
     -----
+    The boosting algorithm is described in [1]_.
+
     In order to predict data, use the :func:`convolve` function::
 
     >>> ds = datasets.get_uts()
@@ -935,12 +937,6 @@ def boosting(
     >>> res = boosting('uts', ['a0', 'a1'], 0, 0.5, partitions=10, model='A', ds=ds)
     >>> y_pred = convolve(res.h_scaled, ['a0', 'a1'], ds=ds)
 
-    The boosting algorithm is described in [1]_.
-
-    See Also
-    --------
-    plot.preview_partitions : preview data partitions for cross-validation
-
     References
     ----------
     .. [1] David, S. V., Mesgarani, N., & Shamma, S. A. (2007). Estimating
@@ -955,13 +951,13 @@ def boosting(
         if scale_data == 'inplace':
             scale_in_place = True
         else:
-            raise ValueError(f"scale_data={scale_data!r}")
+            raise ValueError(f"{scale_data=}")
     else:
-        raise TypeError(f"scale_data={scale_data!r}, need bool or str")
+        raise TypeError(f"{scale_data=}, need bool or str")
     # selective_stopping
     selective_stopping = int(selective_stopping)
     if selective_stopping < 0:
-        raise ValueError(f"selective_stopping={selective_stopping}")
+        raise ValueError(f"{selective_stopping=}")
 
     data = DeconvolutionData(y, x, ds, scale_in_place)
     data.apply_basis(basis, basis_window)
