@@ -3,7 +3,7 @@ from .basic import (
     tqdm,
     PickleableDataClass,
     as_list, as_sequence, ask, deprecated, deprecated_attribute, intervals,
-    LazyProperty, keydefaultdict, n_decimals, natsorted,
+    keydefaultdict, n_decimals, natsorted,
     log_level, set_log_level, ScreenHandler,
 )
 from .system import IS_OSX, IS_WINDOWS, user_activity, restore_main_spec
