@@ -8,14 +8,18 @@
 #  - visualizaes Document
 #  - listens to Document changes
 #  - issues commands to Model
+from collections import defaultdict
 from distutils.version import LooseVersion
 from itertools import repeat
 from math import ceil
+from operator import itemgetter
 import re
 from typing import Optional, Sequence, Tuple, Union
 
 import mne
+import matplotlib.figure
 from matplotlib.patches import Rectangle
+from matplotlib.backends.backend_agg import FigureCanvasAgg
 import numpy as np
 from scipy import linalg
 import wx
@@ -23,17 +27,19 @@ from wx.lib.scrolledpanel import ScrolledPanel
 
 from .. import load, plot, fmtxt
 from .._colorspaces import UNAMBIGUOUS_COLORS
-from .._data_obj import Dataset, Factor, NDVar, asndvar, Categorial, Scalar
+from .._data_obj import Dataset, Factor, NDVar, Categorial, Scalar, asndvar, combine
 from .._io.fiff import _picks
 from .._types import PathArg
-from .._utils.parse import POS_FLOAT_PATTERN
+from .._utils.numpy_utils import INT_TYPES
+from .._utils.parse import FLOAT_PATTERN, POS_FLOAT_PATTERN
 from .._utils.system import IS_OSX
-from ..plot._base import AxisData, DataLayer, PlotType
+from ..mne_fixes._version import MNE_VERSION, V0_24
+from ..plot._base import DISPLAY_UNIT, UNIT_FORMAT, AxisData, DataLayer, PlotType
 from ..plot._topo import _ax_topomap
 from .frame import EelbrainDialog
 from .history import Action, FileDocument, FileModel, FileFrame, FileFrameChild
 from .mpl_canvas import FigureCanvasPanel
-from .text import HTMLFrame
+from .text import HTML2Frame as HTMLFrame
 from .utils import Icon, REValidator
 from . import ID
 
@@ -170,7 +176,10 @@ class Document(FileDocument):
         self.callbacks.callback('case_change', index)
 
     def save(self):
-        self.ica.save(self.path)
+        if MNE_VERSION >= V0_24:
+            self.ica.save(self.path, overwrite=True)
+        else:
+            self.ica.save(self.path)
 
 
 class Model(FileModel):
@@ -202,7 +211,7 @@ class ContextMenu(wx.Menu):
         self.i_epoch = i_epoch
 
 
-class SharedToolsMenu:
+class SharedToolsMenu:  # Frame mixin
     # set by FileFrame:
     doc = None
     config = None
@@ -217,6 +226,9 @@ class SharedToolsMenu:
         button = wx.Button(tb, label="Noisy Epochs")
         button.Bind(wx.EVT_BUTTON, self.OnFindNoisyEpochs)
         tb.AddControl(button)
+        button = wx.Button(tb, label="PSD")
+        button.Bind(wx.EVT_BUTTON, self.OnPlotPSD)
+        tb.AddControl(button)
 
     def MakeToolsMenu(self, menu):
         app = wx.GetApp()
@@ -245,13 +257,29 @@ class SharedToolsMenu:
         menu.AppendSubMenu(blmenu, "Baseline")
 
     def OnFindNoisyEpochs(self, event):
-        dlg = FindNoisyEpochsDialog(self, unit=self.doc.epochs_ndvar.info.get('unit', '?'))
+        unit = self.doc.epochs_ndvar.info.get('unit', '<unknown unit>')
+        if unit in DISPLAY_UNIT:
+            display_unit = DISPLAY_UNIT[unit]
+            scale_factor = 1 / UNIT_FORMAT[display_unit]
+        else:
+            display_unit = unit
+            scale_factor = None
+        dlg = FindNoisyEpochsDialog(self, unit=display_unit)
         rcode = dlg.ShowModal()
         dlg.Destroy()
         if rcode != wx.ID_OK:
             return
         threshold = float(dlg.threshold.GetValue())
+        threshold_desc = f'{threshold:g} {display_unit}'
+        if scale_factor:
+            threshold *= scale_factor
         apply_rejection = dlg.apply_rejection.GetValue()
+        sort_by_component = dlg.sort_by_component.GetValue()
+        max_ch_ratio = dlg.max_ch_ratio.GetValue()
+        if max_ch_ratio:
+            max_ch_ratio = float(max_ch_ratio)
+        else:
+            max_ch_ratio = 0
         dlg.StoreConfig()
 
         # compute and rank
@@ -259,23 +287,79 @@ class SharedToolsMenu:
             epochs = asndvar(self.doc.apply(self.doc.epochs))
         else:
             epochs = self.doc.epochs_ndvar
-        y = epochs.extrema(('time', 'sensor')).abs().x
+        peaks = epochs.extrema(('time', 'sensor')).abs().x
 
         # collect output
-        res = [(i, v) for i, v in enumerate(y) if v >= threshold]  # epoch, value
+        res = [(i, peak) for i, peak in enumerate(peaks) if peak >= threshold]  # epoch, value
         if len(res) == 0:
-            wx.MessageBox(f"No epochs with signals exceeding {threshold} were found.", "No Noisy Epochs Found", style=wx.ICON_INFORMATION)
+            wx.MessageBox(f"No epochs with signals exceeding {threshold_desc} were found.", "No Noisy Epochs Found", style=wx.ICON_INFORMATION)
             return
 
+        if sort_by_component:
+            res_by_component = defaultdict(list)
+            # Find contribution of each component
+            component_magnitude = self.doc.components.abs().sum('sensor')
+            if apply_rejection:
+                component_magnitude.x *= self.doc.accept
+            magnitude = self.doc.sources.abs().sum('time') * component_magnitude
+            for i, peak in res:
+                magnitude_i = magnitude[i]
+                c_max = magnitude_i.argmax()
+                ratio = magnitude_i[c_max] / magnitude_i.sum()
+                res_by_component[c_max].append((i, peak, ratio))
+            # Sort epochs by ratio
+            for res_list in res_by_component.values():
+                res_list.sort(key=itemgetter(2), reverse=True)
+            # Sort components by max ratio
+            max_ratio = {component: values[0][2] for component, values in res_by_component.items()}
+            sorted_components = sorted(max_ratio, key=lambda c: max_ratio[c], reverse=True)
+            res_by_component = {c: res_by_component[c] for c in sorted_components}
+        else:
+            res_by_component = None
+
         # format output
         doc = fmtxt.Section("Noisy epochs")
-        doc.add_paragraph(f"Epochs with signal exceeding {threshold}:")
+        doc.add_paragraph(f"Epochs with signal exceeding {threshold_desc}")
+        if sort_by_component:
+            doc.add_paragraph(f"Sorted by dominant component")
         doc.append(fmtxt.linebreak)
-        for i, value in res:
-            doc.append(fmtxt.Link(self.doc.epoch_labels[i], f'epoch:{i}'))
-            doc.append(f": {value:g}")
-            doc.append(fmtxt.linebreak)
-        InfoFrame(self, "Noisy Epochs", doc, 300, 900)
+        if sort_by_component:
+            for component, values in res_by_component.items():
+                # test whether this is a single noisy channel
+                channel_values = np.sort(np.abs(self.doc.components[component].x))
+                max_channel_ratio = channel_values[-1] / channel_values[-2]
+                if max_ch_ratio and max_channel_ratio > max_ch_ratio:
+                    continue
+                # plot component map
+                figure = matplotlib.figure.Figure(figsize=(1, 1))
+                canvas = FigureCanvasAgg(figure)
+                axes = figure.add_subplot()
+                plot.Topomap(self.doc.components[component], axes=axes)
+                image = fmtxt.Image(f'#{component}', 'jpg')
+                canvas.print_jpeg(image)
+                # Component properties
+                # sec = doc.add_section(f"#{component}")
+                sec = doc
+                heading = fmtxt.FMTextElement(f"#{component}", 'h2')
+                table = fmtxt.Table('lll', rules=False)
+                table.cells(image, heading, f'{max_channel_ratio:.1f}')
+                sec.add_paragraph(table)
+                # sec.add_paragraph([image, f"Ch 1/2 ratio: {max_channel_ratio:.1f}", fmtxt.linebreak])
+                # add links to epochs
+                by_ratio = defaultdict(list)
+                for i, peak, ratio in values:
+                    by_ratio[f'{ratio:.0%}'].append(i)
+                for ratio, epochs in by_ratio.items():
+                    sec.append(f'{ratio}: ')
+                    for i in epochs:
+                        sec.append([fmtxt.Link(self.doc.epoch_labels[i], f'component:{component} epoch:{i}'), ', '])
+                    sec.append(fmtxt.linebreak)
+        else:
+            for i, peak in res:
+                doc.append(fmtxt.Link(self.doc.epoch_labels[i], f'epoch:{i}'))
+                doc.append(f": {peak:g}")
+                doc.append(fmtxt.linebreak)
+        InfoFrame(self, "Noisy Epochs", doc, 300)
 
     def OnFindRareEvents(self, event):
         dlg = FindRareEventsDialog(self)
@@ -317,7 +401,7 @@ class SharedToolsMenu:
             doc.append(f"{c} ({ft:.1f}):  ")
             doc.append(fmtxt.delim_list((fmtxt.Link(self.doc.epoch_labels[e], f'component:{c} epoch:{e}') for e in epochs)))
             doc.append(fmtxt.linebreak)
-        InfoFrame(self, "Rare Events", doc, 500, 900)
+        InfoFrame(self, "Rare Events", doc, 500)
 
     def OnPlotButterfly(self, event):
         self.PlotConditionAverages(self)
@@ -325,6 +409,9 @@ class SharedToolsMenu:
     def OnPlotGrandAverage(self, event):
         self.PlotEpochButterfly()
 
+    def OnPlotPSD(self, event):
+        self.PlotPSD()
+
     def OnSetButterflyBaseline(self, event):
         self.butterfly_baseline = event.GetId()
 
@@ -397,6 +484,15 @@ class SharedToolsMenu:
         else:
             plot.TopoButterfly([original, clean], title=title, axtitle=("Original", "Cleaned"))
 
+    def PlotPSD(self):
+        ds_original = Dataset({'psd': asndvar(self.doc.epochs).fft().mean('sensor')})
+        ds_original[:, 'data'] = 'Source'
+        ds_clean = Dataset({'psd': asndvar(self.doc.apply(self.doc.epochs)).fft().mean('sensor')})
+        ds_clean[:, 'data'] = 'Cleaned'
+        ds = combine((ds_original, ds_clean))
+        colors = {'Source': 'red', 'Cleaned': 'blue'}
+        plot.UTSStat('psd', 'data', ds=ds, error=np.std, w=8, title="Spectrum (±1 STD)", colors=colors)
+
 
 class Frame(SharedToolsMenu, FileFrame):
     """GIU for selecting ICA sensor-space components
@@ -519,7 +615,7 @@ class Frame(SharedToolsMenu, FileFrame):
 
     def CaseChanged(self, index):
         "Update the state of the segments on the current page"
-        if isinstance(index, int):
+        if isinstance(index, INT_TYPES):
             index = [index]
         elif isinstance(index, slice):
             start = index.start or 0
@@ -560,7 +656,7 @@ class Frame(SharedToolsMenu, FileFrame):
         y = source - source.mean()
         y **= 2
         ss = y.sum('time')  # ndvar has epoch as index
-        self.GoToComponentEpoch(epoch=ss.argmax())
+        self.GoToComponentEpoch(i_comp, ss.argmax())
 
     def GoToComponentEpoch(self, component: int = None, epoch: int = None):
         if not self.source_frame:
@@ -613,6 +709,9 @@ class Frame(SharedToolsMenu, FileFrame):
         if n_h >= 2 and n_h != self.n_h:
             self.plot()
 
+    def OnPlotCompFFT(self, event):
+        self.PlotCompFFT(event.EventObject.i_comp)
+
     def OnPlotCompSourceArray(self, event):
         self.PlotCompSourceArray(event.EventObject.i_comp)
 
@@ -625,10 +724,9 @@ class Frame(SharedToolsMenu, FileFrame):
         except RuntimeError:
             return  # can be called after the window closes (Windows)
         if event.inaxes:
-            sb.SetStatusText("#%i of %i ICA Components" %
-                             (event.inaxes.i, len(self.doc.components)))
+            sb.SetStatusText(f"#{event.inaxes.i} of len(self.doc.components) ICA Components")
         else:
-            sb.SetStatusText("%i ICA Components" % len(self.doc.components))
+            sb.SetStatusText(f"{len(self.doc.components)} ICA Components")
 
     def OnRankEpochs(self, event):
         i_comp = event.EventObject.i_comp
@@ -647,7 +745,7 @@ class Frame(SharedToolsMenu, FileFrame):
             link = fmtxt.Link(self.doc.epoch_labels[i], f'component:{i_comp} epoch:{i}')
             lst.add_item(link + f': {ss[i]:.1f}')
         doc = fmtxt.Section(f"#{i_comp} Ranked Epochs", lst)
-        InfoFrame(self, f"Component {i_comp} Epoch SS", doc, 200, 900)
+        InfoFrame(self, f"Component {i_comp} Epoch SS", doc, 200)
 
     def _context_menu(self, i_comp: int = None, i_epoch: int = None):
         menu = ContextMenu(i_comp, i_epoch)
@@ -660,6 +758,8 @@ class Frame(SharedToolsMenu, FileFrame):
             self.Bind(wx.EVT_MENU, self.OnPlotCompTopomap, item)
             item = menu.Append(wx.ID_ANY, "Plot Source Array")
             self.Bind(wx.EVT_MENU, self.OnPlotCompSourceArray, item)
+            item = menu.Append(wx.ID_ANY, "Plot Source FFT")
+            self.Bind(wx.EVT_MENU, self.OnPlotCompFFT, item)
         if i_comp is not None and i_epoch is not None:
             menu.AppendSeparator()
         if i_epoch is not None:
@@ -685,8 +785,7 @@ class Frame(SharedToolsMenu, FileFrame):
         event.Enable(False)
 
     def PlotCompFFT(self, i_comp):
-        plot.UTS(self.doc.sources.sub(component=i_comp).fft().mean('case'),
-                 w=8, title="# %i Frequency Spectrum" % i_comp, legend=False)
+        plot.UTSStat(self.doc.sources.sub(component=i_comp).fft(), error=np.std, w=8, title=f"# {i_comp} Spectrum (±1 STD)", legend=False)
 
     def PlotCompSourceArray(self, i_comp):
         x = self.doc.sources.sub(component=i_comp)
@@ -1225,29 +1324,50 @@ class SourceFrame(SharedToolsMenu, FileFrameChild):
 
 
 class FindNoisyEpochsDialog(EelbrainDialog):
-    _default_threshold = 1.5e-12
+    _default_thresholds = {'µV': 100, 'fT': 1000}
 
-    def __init__(self, parent, *args, unit, **kwargs):
-        super(FindNoisyEpochsDialog, self).__init__(parent, wx.ID_ANY, "Find Bad Epochs", *args, **kwargs)
+    def __init__(self, parent, unit, **kwargs):
+        self.unit = unit
+        super(FindNoisyEpochsDialog, self).__init__(parent, wx.ID_ANY, "Find Bad Epochs", **kwargs)
         config = parent.config
-        threshold = config.ReadFloat("FindNoisyEpochsDialog/threshold", self._default_threshold)
+        threshold = config.ReadFloat(f"FindNoisyEpochsDialog/threshold_{unit}", self._default_threshold())
         apply_rejection = config.ReadBool("FindNoisyEpochsDialog/apply_rejection", True)
+        sort_by_component = config.ReadBool("FindNoisyEpochsDialog/sort_by_component", True)
+        max_ch_ratio = config.Read(f"FindNoisyEpochsDialog/max_ch_ratio", '')
 
         sizer = wx.BoxSizer(wx.VERTICAL)
 
         # Threshold
-        sizer.Add(wx.StaticText(self, label=f"Threshold for bad epochs [{unit}]:"))
+        h_sizer = wx.BoxSizer(wx.HORIZONTAL)
+        h_sizer.Add(wx.StaticText(self, label="Threshold for bad epochs: "))
         validator = REValidator(POS_FLOAT_PATTERN, "Invalid entry: {value}. Please specify a number > 0.", False)
-        self.threshold = ctrl = wx.TextCtrl(self, value=str(threshold), validator=validator)
+        self.threshold = ctrl = wx.TextCtrl(self, value=f'{threshold:g}', validator=validator, style=wx.TE_RIGHT)
         ctrl.SetHelpText("Find epochs in which the signal exceeds this value at any sensor")
         ctrl.SelectAll()
-        sizer.Add(ctrl)
+        h_sizer.Add(ctrl)
+        h_sizer.Add(wx.StaticText(self, label=unit))
+        sizer.Add(h_sizer)
 
         # Apply rejection before finding noisy epochs
         self.apply_rejection = ctrl = wx.CheckBox(self, label="Apply ICA rejection")
         ctrl.SetValue(apply_rejection)
         sizer.Add(ctrl)
 
+        # Sort noisy epochs by component
+        self.sort_by_component = ctrl = wx.CheckBox(self, label="Sort by ICA component")
+        ctrl.SetValue(sort_by_component)
+        sizer.Add(ctrl)
+
+        # Filter by ch 1/2 ratio
+        h_sizer = wx.BoxSizer(wx.HORIZONTAL)
+        h_sizer.Add(wx.StaticText(self, label="Max channel ratio filter: "))
+        validator = REValidator(FLOAT_PATTERN, "Invalid entry: {value}. Please specify a number ≥ 0.", True)
+        self.max_ch_ratio = ctrl = wx.TextCtrl(self, value=max_ch_ratio, validator=validator, style=wx.TE_RIGHT)
+        ctrl.SetHelpText("Filter components that are due to bad channels through the first / second channel ratio")
+        ctrl.SelectAll()
+        h_sizer.Add(ctrl)
+        sizer.Add(h_sizer)
+
         # default button
         btn = wx.Button(self, wx.ID_DEFAULT, "Default Settings")
         sizer.Add(btn, border=2)
@@ -1269,13 +1389,18 @@ class FindNoisyEpochsDialog(EelbrainDialog):
         self.SetSizer(sizer)
         sizer.Fit(self)
 
+    def _default_threshold(self):
+        return self._default_thresholds.get(self.unit, 1)
+
     def OnSetDefault(self, event):
-        self.threshold.SetValue(f'{self._default_threshold}')
+        self.threshold.SetValue(f'{self._default_threshold()}')
 
     def StoreConfig(self):
         config = self.Parent.config
-        config.WriteFloat("FindNoisyEpochsDialog/threshold", float(self.threshold.GetValue()))
+        config.WriteFloat(f"FindNoisyEpochsDialog/threshold_{self.unit}", float(self.threshold.GetValue()))
         config.WriteBool("FindNoisyEpochsDialog/apply_rejection", self.apply_rejection.GetValue())
+        config.WriteBool("FindNoisyEpochsDialog/sort_by_component", self.sort_by_component.GetValue())
+        config.Write(f"FindNoisyEpochsDialog/max_ch_ratio", self.max_ch_ratio.GetValue())
         config.Flush()
 
 
@@ -1328,15 +1453,17 @@ class FindRareEventsDialog(EelbrainDialog):
 
 class InfoFrame(HTMLFrame):
 
-    def __init__(self, parent, title, doc, w, h):
+    def __init__(self, parent: wx.Window, title: str, doc, w: int, h: int = -1):
         pos, size = self.find_pos(w, h)
         style = wx.MINIMIZE_BOX | wx.MAXIMIZE_BOX | wx.RESIZE_BORDER | wx.CAPTION | wx.CLOSE_BOX | wx.FRAME_FLOAT_ON_PARENT | wx.FRAME_TOOL_WINDOW
-        HTMLFrame.__init__(self, parent, title, doc.get_html(), pos=pos, size=size, style=style)
+        html_doc = fmtxt.make_html_doc(doc)
+        HTMLFrame.__init__(self, parent, title, html_doc, pos=pos, size=size, style=style)
 
     @staticmethod
-    def find_pos(w, h):
+    def find_pos(w: int, h: int):
         display_w, display_h = wx.DisplaySize()
-        h = min(h, display_h - 44)
+        h_max = display_h - 44
+        h = h_max if h <= 0 else min(h, h_max)
         pos = (display_w - w, int(round((display_h - h) / 2)))
         return pos, (w, h)
 
