@@ -69,13 +69,12 @@ from ._utils import ui
 
 
 # types
-FMTextLike = Union['FMTextElement', str, ListType['FMTextLike'], Tuple['FMTextLike']]
+FMTextLike = Union['FMTextElement', str, float, Sequence]  # should be Tuple or List['FMTextLike'], but that causes https://github.com/tox-dev/sphinx-autodoc-typehints/issues/223
 
 ENV = MappingProxyType({})
 preferences = dict(
-                   keep_recent=3,  # number of recent tables to keep in memory
-                   html_tables_in_fig=True,
-                   )
+    html_tables_in_fig=True,
+)
 
 # Tags
 # ----
@@ -397,27 +396,33 @@ def html(text: FMTextLike, env: dict = ENV):
         return str(text)
 
 
-def make_html_doc(body, root, resource_dir=None, title=None, meta=None):
+def make_html_doc(
+        body: FMTextLike,
+        root: str = None,
+        resource_dir: str = None,
+        title: FMTextLike = None,
+        meta: dict = None,
+) -> str:
     """Generate HTML document
 
     Parameters
     ----------
-    body : FMText
+    body
         FMTXT object which should be formatted into an HTML document.
-    root : str
+    root
         Path to the directory in which the HTML file is going to be located.
-    resource_dir : None | str
+    resource_dir
         Name for the directory containing resources like images, relative to
         root. If None, images are embedded.
-    title : None | FMText
+    title
         Document title. The default is to try to infer the title from the body
         or use "Untitled".
-    meta : dict
+    meta
         Meta-information for document head.
 
     Returns
     -------
-    html : str
+    html
         HTML document.
     """
     if title is None:
@@ -437,8 +442,7 @@ def make_html_doc(body, root, resource_dir=None, title=None, meta=None):
 
     env = {'root': root, 'resource_dir': resource_dir}
     txt_body = html(body, env)
-    return _html_doc_template.format(meta=meta, title=title, style=style,
-                                     body=txt_body)
+    return _html_doc_template.format(meta=meta, title=title, style=style, body=txt_body)
 
 
 def tex(text, env=None):
@@ -592,9 +596,8 @@ class FMTextElement:
         self.options = options
 
     def __repr__(self):
-        name = self.__class__.__name__
         args = ', '.join(self._repr_items())
-        return "%s(%s)" % (name, args)
+        return f"{self.__class__.__name__}({args})"
 
     def _repr_items(self):
         items = [repr(self.content)]
@@ -607,9 +610,8 @@ class FMTextElement:
     def __str__(self):
         return self.get_str()
 
-    def _ipython_display_(self):
-        from IPython.display import display, HTML
-        display(HTML(html(self)))
+    def _repr_html_(self):
+        return html(self)
 
     def __add__(self, other):
         if isinstance(other, str) and other == '':
@@ -634,11 +636,17 @@ class FMTextElement:
         "Complete HTML representation"
         txt = self._get_html_core(env)
 
-        if self.tag and self.tag in _HTML_TAGS:
-            tag = _HTML_TAGS[self.tag]
-            return _html_element(tag, txt, env, self.options)
+        tag = None
+        if self.tag:
+            if self.tag in _HTML_TAGS:
+                tag = _HTML_TAGS[self.tag]
+            elif re.match(r'h\d', self.tag):
+                tag = self.tag
 
-        return txt
+        if tag:
+            return _html_element(tag, txt, env, self.options)
+        else:
+            return txt
 
     def _get_html_core(self, env):
         "HTML representation of everything inside the tag"
@@ -761,6 +769,8 @@ class FMText(FMTextElement):
         Formatting tag.
     options : dict
         Options for HTML tags.
+    rasterize
+        Try to rasterize images in ``content``.
 
     Notes
     -----
@@ -779,13 +789,14 @@ class FMText(FMTextElement):
             content: FMTextLike = None,
             tag: str = None,
             options: dict = None,
+            rasterize: bool = False,
     ):
         if content is None:
             content = []
         elif isinstance(content, (list, tuple)):
-            content = [asfmtext(item) for item in content]
+            content = [asfmtext(item, rasterize=rasterize) for item in content]
         else:
-            content = [asfmtext(content)]
+            content = [asfmtext(content, rasterize=rasterize)]
         FMTextElement.__init__(self, content, tag, options)
 
     def append(self, content):
@@ -1730,11 +1741,13 @@ class Table(FMTextElement):
         -----
         Most style options are not implemented.
         """
-        from docx import Document
+        try:
+            from docx import Document
+        except ImportError:
+            raise ImportError("The python-docx package needs to be installed for this function")
 
         if path is None:
-            path = ui.ask_saveas(
-                "Save Text File", filetypes=[("Word Document (*.docx)", "*.docx")])
+            path = ui.ask_saveas("Save Text File", filetypes=[("Word Document (*.docx)", "*.docx")])
         if not path:
             return
         document = Document()
@@ -1747,10 +1760,13 @@ class Table(FMTextElement):
         for row in self.rows:
             if isinstance(row, str):
                 continue
-            d_row = table.add_row()
+            doc_row = table.add_row()
             i = 0
             for cell in row:
-                d_row.cells[i].text = str(cell)
+                doc_cell = doc_row.cells[i]
+                for j in range(i + 1, i + cell.width):
+                    doc_cell.merge(doc_row.cells[j])
+                doc_cell.text = str(cell)
                 i += cell.width
 
         document.save(path)
@@ -1959,10 +1975,18 @@ class Figure(FMText):
 
 
 class FloatingLayout(FMText):
-    "Arrange contents as floating elements"
+    """Arrange contents as floating elements
 
-    def __init__(self, content: FMTextLike):
-        FMText.__init__(self, content)
+    Parameters
+    ----------
+    content
+        Floating layout contents.
+    rasterize
+        Try to rasterize images in ``content``.
+    """
+
+    def __init__(self, content: FMTextLike, rasterize: bool = False):
+        FMText.__init__(self, content, rasterize=rasterize)
         self._options = {
             'display': 'inline-block',
             'margin': '10px',
@@ -1977,7 +2001,7 @@ class FloatingLayout(FMText):
         ))
         items = (item.get_html(env) for item in self.content)
         items = (f'<div class="floating-box">\n{item}\n</div>' for item in items)
-        return '\n\n'.join((header, *items))
+        return '\n\n'.join([header, *items])
 
 
 class Section(FMText):
